
#include "Level.h"
#include "Global.h"
#include "partypack/bitmap.h"
#include "aatext.h"

#include <iostream>

using namespace std;

Level::Level(string _url, string _title, int _level) {
	url = _url;
	title = _title;
	level = _level+1;
	parsed = false;
	unlocked = false;
}

Level::~Level() {
	for (size_t i=0; i<posts.size(); i++)
		delete posts[i];
}

int Level::calcHeight(int width) {
	int height = 150;
	for (size_t i=0; i<posts.size(); i++) {
		Post *p = posts[i];
		height += p->calcHeight();
	}
	height += 50;
	return min(height,4048);
}

void Level::draw(alBitmap &bitmap) {
	bitmap.clear(makecol(255,255,255));
	int xstart = 24;
	bitmap.draw(gSys->rm.load<Bitmap>("data/header.png")->getABitmap(), 0, 0);

	int linkColor = makecol(0x33,0x66,0x99);
	int textColor = makecol(0,0,0);
	int headerColor = makecol(160,208,160);
	int x = xstart;
	int y = 100;

	aatextout(bitmap.getBITMAP(), gSys->v10, "Allegro.cc Forums", x,y, linkColor);
	x += 2 + text_length(gSys->v10, "Allegro.cc Forums");
	aatextout(bitmap.getBITMAP(), gSys->v10, ">", x,y, textColor);
	x += 2 + text_length(gSys->v10, ">");
	aatextout(bitmap.getBITMAP(), gSys->v10, forum.c_str(), x,y, linkColor);
	x += 2 + text_length(gSys->v10, forum.c_str());
	aatextout(bitmap.getBITMAP(), gSys->v10, ">", x,y, textColor);
	x += 2 + text_length(gSys->v10, ">");
	aatextout(bitmap.getBITMAP(), gSys->v10b, title.c_str(), x,y, textColor);

	x = xstart;
	y += 30;

	bitmap.draw(gSys->rm.load<Bitmap>("data/print-icon.png")->getABitmap(), x,y);
	y += 22;

	bitmap.rectFill(x,y,640-x,y+28, headerColor);
	bitmap.rect(x,y,640-x,y+28, textColor);

	aatextout(bitmap.getBITMAP(), gSys->v10b, title.c_str(), x+5,y+6, textColor);

	y+=28;

	for (size_t i=0; i<posts.size(); i++) {
		Post *p = posts[i];
		y += p->draw(y, &bitmap);
	}
}

int Level::Post::calcHeight() {
	return draw(0,0);
}
int Level::Post::draw(int yy, alBitmap *bitmap) {
	int x=0,y=yy;

	int width = 640 - 24 - 24 - 150 - 10;
	FONT *font = gSys->v10;
	int color = makecol(0,0,0);
	int y2 = y;

	if (bitmap) {
		int height = draw(0,0)+50;
		bitmap->rectFill(24,y,640-24,y+height, makecol(247,247,247));
		bitmap->rect(24,y,640-24,y+height, color);
	}
	if (bitmap)
		aatextout(bitmap->getBITMAP(), gSys->v10b, author.c_str(), 5 + 24 + x,y2, color);
	y2+=text_height(gSys->v10b);
	if (bitmap)
		aatextout(bitmap->getBITMAP(), gSys->v9, number.c_str(), 5 + 24 + x,y2, color);
	y2+=text_height(gSys->v9);
	if (bitmap)
		aatextout(bitmap->getBITMAP(), gSys->v9, joindate.c_str(), 5 + 24 + x,y2, color);
	y2+=text_height(gSys->v9) + 30;

	if (hasAvatar) {
		if (bitmap) {
			string file = "downloads/"+avatar;
			Bitmap *b = gSys->rm.load<Bitmap>(file);
			if (!b) {
				gSys->curl.download("http://www.allegro.cc/members/avatars/"+avatar, file);
				b = gSys->rm.load<Bitmap>(file);
			}
			set_alpha_blender();
			if (b) bitmap->draw(b->getABitmap(), 50, y2);
		}
		y2 += 100;
	}

	y+=5;

	for (size_t j=0; j<body.size(); j++) {
		string s = body[j];

		if (s[0]=='<') {
			if (s=="<br") {
				x = 0;
				y += text_height(font);
			} else if (s=="<b") {
				if (font==gSys->v10)
					font = gSys->v10b;
				else if (font==gSys->v10i)
					font = gSys->v10bi;
			} else if (s=="<i") {
				if (font==gSys->v10)
					font = gSys->v10i;
				else if (font==gSys->v10b)
					font = gSys->v10bi;
			} else if (s=="</b") {
				if (font==gSys->v10b)
					font = gSys->v10;
				else if (font==gSys->v10bi)
					font = gSys->v10i;
			} else if (s=="</i") {
				if (font==gSys->v10i)
					font = gSys->v10;
				else if (font==gSys->v10bi)
					font = gSys->v10b;
			} else if (s=="<a") {
				color = makecol(0x33,0x66,0x99);
			} else if (s=="</a") {
				color = makecol(0,0,0);
			} else if (s=="<sig") {
				y += text_height(font) + 3;
				x = 0;
				font = gSys->v8;
			}
		} else {
			string ss;
			for (size_t i=0; i<=s.size(); i++) {
				if (i==s.size() || isspace(s[i])) {
					int len = text_length(font, ss.c_str());
					if (x+len>width) {
						x = 0;
						y += text_height(font);
					}
					if (bitmap)
						aatextout(bitmap->getBITMAP(), font, ss.c_str(), 150 + 24 + x,y, color);
					x += len + 3;
					ss="";
				} else
					ss+=s[i];
			}
		}
	}
	y+=text_height(font);
	if (y2>y)
		y = y2;
	y+=5;
	return y-yy;
}

void Level::parse() {
	if (parsed) return;

	string file = "downloads/" + url;

	gSys->curl.download("http://www.allegro.cc/forums/view_thread.php?_id=" + url,file);


	FILE *f = fopen(file.c_str(),"r");
	if (f) {
		int c = fgetc(f),lc=c;
		string s,s2;
		int sc;
		string url;
		int state = 0;
		Post *post = new Post;
		while (c!=EOF) {
			switch (state) {
				case 0:
					if (c=='<') {
						state = 1;
						s = "";
					}
					break;
				case 1:
					s += c;
					if (s=="title>") {
						state = 2;
						s = "";
						break;
					} else if (s == "div class='originator'>") {
						post = new Post;
						posts.push_back(post);
						state = 3;
						s = "";
						break;
					} else if (s == "div class='member_number'>") {
						state = 4;
						s = "";
						break;
					} else if (s == "div class='join_date'>") {
						state = 5;
						s = "";
						break;
					} else if (s == "img src='/members/avatars/") {
						post->hasAvatar = true;
						state = 6;
						s = "";
						break;
					} else if (s == "a title=\"Right-click to copy the link location\"") {
						state = 7;
						s = "";
						break;
					} else if (s == "font size='2'>") {
						state = 9;
						s = "";
						break;
					}

					if (c=='>') {
						state = 0;
						s = "";
					}


					break;
				case 2:
					if (c==':') {
						forum = s;
						s = "";
						state = 0;
					}
					s+=c;
					if (s=="Allegro.cc - ")
						s="";
					break;
				case 3:
				case 4:
				case 5:
				case 8:
					if (c=='<') {
						switch (state) {
							case 3:
								post->author = s;
								if (s=="Marcello")
									unlocked = true;
								break;
							case 4:
								post->number = s;
								break;
							case 5:
								post->joindate = s;
								break;
							case 8:
								post->postedon = s;
								break;
						}

						s = "";
						state = 1;
					}
					s+=c;
					break;
				case 6:
					if (c=='\'') {
						post->avatar = s;
						s = "";
						state = 1;
					}
					s+=c;
					break;
				case 7:
					if (c=='>')
						state = 8;
					break;
				case 9:
					if (c=='<') {
						state = 10;
						if (s.size()>0)
							post->body.push_back(s);
						s = "";
						break;
					} else if (c=='&') {
						state = 11;
						s2 = "";
						break;
					}
					if (isspace(c)) {
						if (s.size()>0 && !isspace(lc))
							s+=' ';
					} else
						s+=c;
					break;
				case 10:
					s += c;
					if (s=="br />") {
						post->body.push_back("<br");
					} else if (s=="b>") {
						post->body.push_back("<b");
					} else if (s=="i>") {
						post->body.push_back("<i");
					} else if (s=="/b>") {
						post->body.push_back("</b");
					} else if (s=="/i>") {
						post->body.push_back("</i");
					} else if (s=="a href=") {
						post->body.push_back("<a");
					} else if (s=="/a>") {
						post->body.push_back("</a");
					} else if (s=="/i>") {
						post->body.push_back("</i");
					} else if (s=="div class='signature'>") {
						post->body.push_back("<sig");
					} else if (s=="/font>") {
						state = 0;
						s="";
						break;
					}
					if (c=='>') {
						state = 9;
						s = "";
					}
					break;
				case 11:
					if (c==';') {
						state = 9;
						break;
					}
					if (c=='#') {
						state = 12;
						sc = 0;
					}
					if (s2=="nbsp") {
						s+=' ';
					} else if (s2=="quot") {
						s+='"';
					} else if (s2=="lt") {
						s+='<';
					} else if (s2=="gt") {
						s+='>';
					}
					s2 += c;
					break;
				case 12:
					if (c==';') {
						if (sc>=0x20 && sc<=0x7F)
							s+=(char)sc;
						state = 9;
					}
					if (c>='0' && c<='9') {
						sc *= 10;
						sc += c - '0';
					}
					break;

			}



			lc = c;
			c = fgetc(f);
		}
		fclose(f);
	}

	parsed = true;
}
