#include <cmath>

#include "Global.h"

#include <iostream>
#include "MainGame.h"
#include "Menu.h"
#include "alBitmap.h"
#include "Random.h"
#include "Audio.h"
#include "partypack/vector2d.h"
#include "partypack/sprite_animator.h"
#include "partypack/bitmap.h"
#include "Level.h"
#include "LevelSet.h"

using namespace std;

static void switch_in() {
	gSys->maingame->paused = false;
}
static void switch_out() {
	gSys->maingame->paused = true;
}

MainGame::MainGame() {
	score = 0;
	set_display_switch_callback(SWITCH_IN, switch_in);
	set_display_switch_callback(SWITCH_OUT, switch_out);
}
MainGame::~MainGame() {
	for (vector<Troll*>::iterator ii = trolls.begin(); ii != trolls.end(); ii++)
		delete (*ii);
	for (list<Fire*>::iterator ii = flames.begin(); ii != flames.end(); ii++)
		delete (*ii);
	for (list<Explosion*>::iterator ii = explosions.begin(); ii != explosions.end(); ii++)
		delete (*ii);
}

void MainGame::setup(Level *level, bool machine) {
	level->parse();

	int leveln = level->getLevel();

	background.create(640,max(640,level->calcHeight(640)));

	level->draw(background);

	machine = level->getUnlocked();

	flame = gSys->rm.load<Bitmap>("data/flame.png")->getABitmap();
	flamesmall = gSys->rm.load<Bitmap>("data/flamesmall.png")->getABitmap();
	smoke = gSys->rm.load<Bitmap>("data/smoke.png")->getABitmap();
	pin = gSys->rm.load<Bitmap>("data/pin2.png")->getABitmap();
	gunshotfx = gSys->rm.load<Audio>("data/shoot.wav")->getSAMPLE();
	reloadfx = gSys->rm.load<Audio>("data/reload.wav")->getSAMPLE();
	hitfx = gSys->rm.load<Audio>("data/hit.wav")->getSAMPLE();
	explodefx = gSys->rm.load<Audio>("data/explode.wav")->getSAMPLE();

	trollsn = trollsleft = leveln*3;

	for (vector<Troll*>::iterator ii = trolls.begin(); ii != trolls.end(); ii++)
		delete (*ii);

	trolls.clear();
	missedpins.clear();


	for (int i=0; i<trollsn; i++)
		trolls.push_back(new Troll(Random::intInRange(40,600),Random::intInRange(40,background.getHeight()-40),this));

	crosshair = vector2f(320,240);
	scroll = vector2f(0,0);

	pins = (machine ? 40 : 5) * trollsn;
	activepins = 4;
	lastshot = 0;

	shots = hits = misses = 0;

	time = trollsn * 10 * 100;

	paused = false;
	canPlay = true;
	machinegun = false;
	hasmachine = machine;
}
void MainGame::update() {

	if (!paused && canPlay) {
		for (vector<Troll*>::iterator ii = trolls.begin(); ii != trolls.end(); ii++)
			(*ii)->update();

		background.acquire();

		set_multiply_blender(255,255,255,255);
		for (list<Fire*>::iterator ii = flames.begin(); ii != flames.end();) {
			if (!(*ii)->update()) {
				background.drawTrans(smoke,(int)((*ii)->pos.x)-16,(int)((*ii)->pos.y)-16);
				delete (*ii);
				ii = flames.erase(ii);
			} else
				ii++;
		}
		for (list<Explosion*>::iterator ii = explosions.begin(); ii != explosions.end();) {
			if (!(*ii)->update()) {
				delete (*ii);
				ii = explosions.erase(ii);
			} else
				ii++;
		}
		background.release();


		int mickeyx,mickeyy;
		get_mouse_mickeys(&mickeyx, &mickeyy);
		position_mouse(320,240);
		crosshaird += vector2f(mickeyx*0.2,mickeyy*0.2);
		crosshair += crosshaird;
		if (crosshaird.magnitude()>0.2)
			crosshaird *= 0.85;
		crosshair.clamp(vector2f(2,2),vector2f(638,478));

		if (crosshair.y<100) {
			scroll.y -= (100-crosshair.y)*0.1f;
			if (scroll.y<0) scroll.y = 0;
		} else if (crosshair.y>380) {
			scroll.y += (crosshair.y-380)*0.1f;
			if (scroll.y>background.getHeight()-480) scroll.y = background.getHeight()-480;
		}

		static int pressm = 0;

		if ((mouse_b&1) && lastshot == 0 && (machinegun || !pressm)) {
			pressm = 1;
			if (activepins==0)
				reload();
			else {
				play_sample(gunshotfx, 255, mouse_x*255/640, 1000, 0);
				bool hit = false;
				for (vector<Troll*>::iterator ii = trolls.begin(); ii != trolls.end(); ii++)
					if ((*ii)->shoot(scroll+crosshair)) {
						hit = true;
						break;
					}
				if (hit) {
					hits++;
				} else {
					misses++;
					missedpins.push_back(Pin(scroll+crosshair,this));
				}
				shots++;

				activepins--;
				lastshot = machinegun ? 10 : 50;
			}
		}
		if (!(mouse_b&1))
			pressm = 0;
		if ((key[KEY_SPACE] || (mouse_b&2)) && lastshot == 0)
			reload();

		if (key[KEY_1]) {
			if (activepins>4) {
				pins+=activepins-4;
				activepins=4;
			}
			machinegun = false;
		}
		if (hasmachine&&key[KEY_2])
			machinegun = true;
		if (lastshot>0)
			lastshot--;
		time--;
	}
	canPlay = time>0 && trollsleft>0;
	if (canPlay) {
		static int pressp = 0;
		if (key[KEY_P]) {
			if (!pressp) paused = !paused;
			pressp = 1;
		} else
			pressp = 0;
	} else {
		if (key[KEY_ENTER]) {
			if (trollsleft == 0) {
				score+=time+hits*hits*hits*1000/shots/shots/shots;
				gSys->levels->nextLevel();
			} else {
				score = 0;
				gSys->levels->restart();
			}
			gSys->game = gSys->menu;
			clear_keybuf();
		}
	}

}
void MainGame::reload() {
	int count = machinegun ? 16 : 4;
	int newammo = min(count-activepins,pins);
	if (newammo==0) return;
	pins -= newammo;
	activepins+=newammo;
	play_sample(reloadfx, 255, 128, 1000, 0);

	lastshot = machinegun ? 25 : 100;
}
void MainGame::offset(vector2f pos, bool inscope, vector2f &v, bool &b) {
	v = pos - scroll - crosshair;
	b = v.x>-50 && v.x<50 && v.y>-50 && v.y<50 && inscope;
	if (b) {
		v *= 4;
		v += crosshair;
	} else
		v = pos - scroll;
}
void MainGame::draw(alBitmap &bitmap) {
	bitmap.draw(background,0,-(int)scroll.y);

	int cx = (int)crosshair.x;
	int cy = (int)crosshair.y;

	for (vector<Pin>::iterator ii = missedpins.begin(); ii != missedpins.end(); ii++)
		(*ii).draw(bitmap,vector2f(0,0),false);

	for (vector<Troll*>::iterator ii = trolls.begin(); ii != trolls.end(); ii++)
		(*ii)->draw(bitmap,false);

	set_multiply_blender(255,255,255,255);
	for (list<Fire*>::iterator ii = flames.begin(); ii != flames.end(); ii++)
		(*ii)->draw(bitmap,false);
	for (list<Explosion*>::iterator ii = explosions.begin(); ii != explosions.end(); ii++)
		(*ii)->draw(bitmap,false);




	//
	bitmap.setClip(cx-100,cy-100,cx+100,cy+100);
	bitmap.rectFill(cx-100,cy-100,cx+100,cy+100,makecol(30,30,30));
	int x1 = cx-26;
	int y1 = cy-26+(int)scroll.y;
	//if (x1>=0 && y1>=0 && x1+52<background.getWidth() && y1+52<background.getHeight())
		background.stretchDrawTo(bitmap,
						cx-100-(int)((crosshair.x-cx)*4),cy-100-(int)((crosshair.y-cy)*4),
						x1,y1,
						52,52,
						208,208);


	for (vector<Pin>::iterator ii = missedpins.begin(); ii != missedpins.end(); ii++)
		(*ii).draw(bitmap, vector2f(0,0),true);

	for (vector<Troll*>::iterator ii = trolls.begin(); ii != trolls.end(); ii++)
		(*ii)->draw(bitmap,true);

	//set_multiply_blender(255,255,255,255);
	for (list<Fire*>::iterator ii = flames.begin(); ii != flames.end(); ii++)
		(*ii)->draw(bitmap,true);
	for (list<Explosion*>::iterator ii = explosions.begin(); ii != explosions.end(); ii++)
		(*ii)->draw(bitmap,true);


	bitmap.setClip(0,0,640,480);
	bitmap.hLine(0,cy,640,makecol(0,0,0));
	bitmap.vLine(cx,0,480,makecol(0,0,0));
	bitmap.rect(cx-100,cy-100,cx+100,cy+100,makecol(0,0,0));
	for (int i=0; i<min(8,activepins); i++)
		bitmap.draw(pin, 460+i*21-28, 380+60);
	for (int i=0; i<activepins-8; i++)
		bitmap.draw(pin, 460+i*21-28, 380+30);
	bitmap.textprintf(gSys->futura, 271, 436, makecol(255,255,255), -1, "Ammo: %i", pins);
	bitmap.textprintf(gSys->futura, 270, 435, makecol(0,0,0), -1, "Ammo: %i", pins);


	bitmap.textprintf(gSys->futura, 11, 6, makecol(255,255,255), -1, "Score: %i", score);
	bitmap.textprintf(gSys->futura, 10, 5, makecol(0,0,0), -1, "Score: %i", score);
	bitmap.textprintf(gSys->futura16, 11, 41, makecol(255,255,255), -1, "Trolls: %i", trollsleft);
	bitmap.textprintf(gSys->futura16, 10, 40, makecol(0,0,0), -1, "Trolls: %i", trollsleft);

	bitmap.textprintf(gSys->futura, 401, 6, makecol(255,255,255), -1, "Time: %i:%02i.%02i", time/100/60, (time/100)%60, time%100);
	bitmap.textprintf(gSys->futura, 400, 5, makecol(0,0,0), -1, "Time: %i:%02i.%02i", time/100/60, (time/100)%60, time%100);

	if (hasmachine) {
		bitmap.textprintf(gSys->futura16, 11, 421, makecol(255,255,255), -1, "Machinegun unlocked!");
		bitmap.textprintf(gSys->futura16, 10, 420, makecol(0,0,0), -1, "Machinegun unlocked!");

		bitmap.textprintf(gSys->futura16, 11, 451, makecol(255,255,255), -1, "Press 1/2 to switch.");
		bitmap.textprintf(gSys->futura16, 10, 450, makecol(0,0,0), -1, "Press 1/2 to switch.");
	}
	if (paused) {
		for (int y=0; y<480; y+=2)
			bitmap.hLine(0,y,640,makecol(0,0,0));
		bitmap.rectFill(230,220,410,255,makecol(255,255,255));
		bitmap.rect    (230,220,410,255,makecol(0,0,0));
		bitmap.textoutCenter(gSys->futura,"Paused",320,217,makecol(0,0,0));
	}
	if (!canPlay) {
		for (int y=0; y<480; y+=2)
			bitmap.hLine(0,y,640,makecol(0,0,0));
		for (int x=0; x<640; x+=2)
			bitmap.vLine(x,0,480,makecol(0,0,0));
		bool win = trollsleft == 0;
		int y = 117;
		bitmap.textoutCenter(gSys->futura,win?"Level Complete":"Game Over",320,y,makecol(255,255,255));
		y += text_height(gSys->futura) + 10;
		bitmap.textprintfCenter(gSys->futura16,320,y,makecol(255,255,255),-1,"Shots fired: %i",shots);
		y += text_height(gSys->futura16);
		bitmap.textprintfCenter(gSys->futura16,320,y,makecol(255,255,255),-1,"Hits: %i",hits);
		y += text_height(gSys->futura16);
		bitmap.textprintfCenter(gSys->futura16,320,y,makecol(255,255,255),-1,"Accuracy: %.2f", (float)(hits*100)/shots);
		y += text_height(gSys->futura16);
		bitmap.textprintfCenter(gSys->futura16,320,y,makecol(255,255,255),-1,"Time left: %i:%02i.%02i", time/100/60, (time/100)%60, time%100);
		y += text_height(gSys->futura16);
		bitmap.textprintfCenter(gSys->futura16,320,y,makecol(255,255,255),-1,"Time bonus: %i", time/3);
		y += text_height(gSys->futura16);
		bitmap.textprintfCenter(gSys->futura16,320,y,makecol(255,255,255),-1,"Accuracy bonus: %i", hits*hits*hits*1000/shots/shots/shots);
		y += text_height(gSys->futura16)*2;
		bitmap.textprintfCenter(gSys->futura16,320,y,makecol(255,255,255),-1,"Press enter...");
	}
}

MainGame::Troll::Troll(int x, int y, MainGame *_game) {
	anim = new SpriteAnimator(gSys->rm.load<Sprite>("data/troll.nws"));
	animsmall = new SpriteAnimator(gSys->rm.load<Sprite>("data/trollsmall.nws"));
	animhit = new SpriteAnimator(gSys->rm.load<Sprite>("data/trollhit.nws"));

	anim->play(1,true);
	animsmall->play(1,true);
	animhit->play(1,true);

	pos.x=x;
	pos.y=y;
	game = _game;
	delta.x = Random::floatIn1() - 0.5;
	delta.y = Random::floatIn1() - 0.5;
	frame = 0;
	health = 100;
	r = 0;
	alive = true;
}
MainGame::Troll::~Troll() {
	if (anim) delete anim;
}

void MainGame::Troll::update() {
	anim->update();
	animhit->update();
	animsmall->update();
	pos += delta;
	if (pos.x<10) delta.x = Random::floatIn1();
	if (pos.x>630) delta.x = -Random::floatIn1();
	if (pos.y<10) delta.y = Random::floatIn1();
	if (pos.y>game->background.getHeight()-10) delta.y = -Random::floatIn1();

	if (health>0) {
		delta.x+=(float)sin(r)*0.003f;
		delta.y+=(float)cos(r)*0.003f;
		r+=0.01f;
		anim->setMirror(delta.x<0);
		animsmall->setMirror(delta.x<0);
		animhit->setMirror(delta.x<0);
		game->flames.push_back(new Fire(pos.x+(delta.x<0 ? -10 : 10),pos.y-10,delta.x<0 ? delta.x-3 : delta.x+3,game));
	}
}
bool MainGame::Troll::shoot(vector2f p) {

	p -= pos;
	int ix = (int)(p.x*4)+40;
	int iy = (int)(p.y*4)+99;
	BITMAP *bmp = anim->getBITMAP();

	if (ix<0||iy<0||ix>=bmp->w||iy>=bmp->h||getpixel(bmp,ix,iy)==bitmap_mask_color(bmp)) return false;

	play_sample(game->hitfx, 255, (int)(p.x+pos.x)*255/640, 1000, 0);

	delta.x *= game->machinegun ? 0.75f : 0.25f;

	if (health>0) {
		bmp = animhit->getBITMAP();
		int points = 0;
		int r = getr(getpixel(bmp,ix,iy));

		if (r>225) // eye shot
			points = 500;
		else if (r>175) // head shot
			points = 250;
		else if (r>125) // heart shot
			points = 150;
		else if (r>75) { // tank shot
			points = 100;
			play_sample(game->explodefx, 255, (int)(p.x+pos.x)*255/640, 1000, 0);
			for (int i=0; i<20; i++)
				game->explosions.push_back(new Explosion(p.x+pos.x+20*Random::floatIn1()-10,p.y+pos.y+20*Random::floatIn1()-10,game));
		} else if (r>25) // body shot
			points = 100;
		else	// everything else
			points = 50;

		if (game->machinegun)
			points /= 6;

		health -= points;

		game->score += points;
	}

	hits.push_back(Pin(p,game));

	if (alive && health<=0) {
		delta.x = delta.y = 0;
		anim->stop();
		animhit->stop();
		animsmall->stop();
		game->trollsleft--;
		alive = false;
	}

	return true;
}
void MainGame::Troll::draw(alBitmap &bitmap, bool inscope) {
	vector2f v; bool b;
	game->offset(pos,inscope,v,b);
	if (b||!inscope) {
		(b?anim:animsmall)->draw(bitmap.getBITMAP(),v);
		for (vector<Pin>::iterator ii = hits.begin(); ii != hits.end(); ii++) {
			(*ii).draw(bitmap,pos,inscope);
		}
	}

}


MainGame::Fire::Fire(float x, float y, float dx, MainGame *_game) : pos(x,y), delta(dx,0) {
	game = _game;
	age = 0;
	pos.x+=Random::floatIn1();
	delta.y = Random::floatIn1()-0.5f;

}

MainGame::Fire::~Fire() {
}

bool MainGame::Fire::update() {
	age++;
	pos+=delta;
	return age<10;
}

void MainGame::Fire::draw(alBitmap &bitmap, bool inscope) {
	vector2f v; bool b;
	game->offset(pos,inscope,v,b);
	if (b) {
		v.x -= 24;
		v.y -= 24;
	} else {
		v.x -= 6;
		v.y -= 6;
	}
	if (b||!inscope)
		bitmap.drawTrans(b?game->flame:game->flamesmall, (int)v.x, (int)v.y);
}
MainGame::Explosion::Explosion(float x, float y, MainGame *_game) : pos(x,y) {
	game = _game;

	delta.x = Random::floatIn1()-0.5f;
	delta.y = Random::floatIn1()-0.5f;
	anim = new SpriteAnimator(gSys->rm.load<Sprite>("data/explode.nws"));
	animsmall = new SpriteAnimator(gSys->rm.load<Sprite>("data/explodesmall.nws"));

	anim->play(1,false);
	animsmall->play(1,false);
	age = 0;

}

MainGame::Explosion::~Explosion() {
}

bool MainGame::Explosion::update() {
	age++;
	pos+=delta;
	return age<50;
}

void MainGame::Explosion::draw(alBitmap &bitmap, bool inscope) {
	vector2f v; bool b;
	game->offset(pos,inscope,v,b);
	if (b||!inscope)
		(b?anim:animsmall)->draw(bitmap.getBITMAP(),v);
}

MainGame::Pin::Pin(vector2f _pos, MainGame *_game) : pos(_pos), game(_game) {
	string file = "data/pin";
	file +=(char)('1'+Random::intIn(5));

	pin = gSys->rm.load<Bitmap>(file + ".png")->getABitmap();
	pinshadow = gSys->rm.load<Bitmap>(file + "b.png")->getABitmap();
	pinsmall = gSys->rm.load<Bitmap>(file + "s.png")->getABitmap();
	pinsmallshadow = gSys->rm.load<Bitmap>(file + "bs.png")->getABitmap();

}
MainGame::Pin::~Pin() {
}
void MainGame::Pin::draw(alBitmap &bitmap, vector2f offset, bool inscope) {
	vector2f v; bool b;
	game->offset(pos+offset,inscope,v,b);
	v.x -= b ? 32 : 8;
	v.y -= b ? 32 : 8;

	set_multiply_blender(255,255,255,255);

	if (b||!inscope)
		bitmap.drawTrans(b?pinshadow:pinsmallshadow, (int)v.x,(int)v.y);

	if (b||!inscope)
		bitmap.draw(b?pin:pinsmall, (int)v.x,(int)v.y);
}

